## $Id: browser.py 16629 2021-09-20 07:16:50Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import hashlib
import grok
from kofacustom.nigeria.interswitch.browser import (
    module_activated,
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPaymentVerifyWebservicePageApplicant,
    InterswitchPaymentVerifyWebservicePageStudent,
    InterswitchPageStudent, InterswitchPageApplicant,
    )
from kofacustom.nigeria.interswitch.paydirectbrowser import (
    PAYDirectPageStudent, PAYDirectPageApplicant,
    StudentRefNumberSlip, ApplicantRefNumberSlip,
    PAYDirectActionButtonApplicant,
    PAYDirectActionButtonStudent
    )
from kofacustom.iuokada.students.interfaces import ICustomStudentOnlinePayment
from kofacustom.iuokada.applicants.interfaces import ICustomApplicantOnlinePayment
from kofacustom.iuokada.interfaces import MessageFactory as _

PRODUCT_ID = '7922'
SITE_NAME = 'iuokada.waeup.org'
PROVIDER_ACCT = '0773411069'
PROVIDER_BANK_ID = '31'
PROVIDER_ITEM_NAME = 'WAeAC'
INSTITUTION_NAME = 'IUOkada'
CURRENCY = '566'
GATEWAY_AMT = 250.0
#MAC = 'CEF793CBBE838AA0CBB29B74D571113B4EA6586D3BA77E7CFA0B95E278364EFC4526ED7BD255A366CDDE11F1F607F0F844B09D93B16F7CFE87563B2272007AB3' # must be provided by Interswitch
MAC = '4D8723F33D728BE3F4A77B2DFB3F57B0BCF2DD818759D54A87B2A60874067F28D94F2B91E29BFB884F920F48E0973D826835A8F2D6F74220C64EDE4DE00E45AA'

POST_ACTION = 'https://webpay.interswitchng.com/paydirect/pay'
#POST_ACTION = 'https://sandbox.interswitchng.com/webpay/pay'
HOST = 'webpay.interswitchng.com'
#HOST = 'sandbox.interswitchng.com'
URL = '/paydirect/api/v1/gettransaction.json'
#URL = '/webpay/api/v1/gettransaction.json'

httplib.HTTPSConnection.debuglevel = 0
HTTPS = True

class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    mac = MAC  
    gateway_amt = GATEWAY_AMT

    def update(self):
        if not module_activated(
            self.context.student.current_session, self.context):
            self.flash(_('Forbidden'), type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        # Already now it becomes an Interswitch payment. We set the net amount
        # and add the gateway amount.
        if not self.context.r_company:
            self.context.net_amt = self.context.amount_auth
            self.context.amount_auth += self.gateway_amt
            self.context.gateway_amt = self.gateway_amt
            self.context.r_company = u'interswitch'
        student = self.student
        xmldict = self.xmldict
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_acct'] = ''
        xmldict['institution_bank_id'] = ''
        provider_amt = 0.0
        self.pay_item_id = ''
        # Institution data
        if self.context.p_category in (
            'schoolfee', 'schoolfee40', 'secondinstal'):
            self.pay_item_id = '101'
        elif self.context.p_category == 'book':
            self.pay_item_id = '103'
        elif self.context.p_category == 'parentsconsult':
            self.pay_item_id = '104'
        elif self.context.p_category in (
            'municipal_fresh', 'municipal_returning',
            'transcript_local', 'transcript_overseas', 'transcript'):
            self.pay_item_id = '105'
        else:
            self.pay_item_id = '102'
        if self.context.p_category in (
            'registration', 'required_combi', 'pg_other', 'jupeb_reg'):
            provider_amt = 5000.0
        if self.context.p_category in (
            'schoolfee', 'schoolfee40') and student.is_jupeb:
            provider_amt = 5000.0
        if self.context.p_category.startswith('jupeb'):
            self.pay_item_id = '102'
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 100 * self.context.net_amt - (
            100 * provider_amt)

        if self.context.p_option == 'access':
            xmldict['institution_acct'] = '0040484781'
            xmldict['institution_bank_id'] = '31'
        elif self.context.p_option == 'first':
            xmldict['institution_acct'] = '2021420049'
            xmldict['institution_bank_id'] = '8'
        elif self.context.p_option == 'zenith':
            xmldict['institution_acct'] = '1011005811'
            xmldict['institution_bank_id'] = '117'
        else:
            xmldict['institution_acct'] = '0040484781'
            xmldict['institution_bank_id'] = '31'

        if provider_amt == 0:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.amount_auth = int(100 * self.context.amount_auth)
        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPageApplicant(InterswitchPageApplicant):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    mac = MAC
    gateway_amt = GATEWAY_AMT

    def update(self):
        if not module_activated(
            self.context.__parent__.__parent__.year, self.context):
            self.flash(_('Forbidden'), type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        # Already now it becomes an Interswitch payment. We set the net amount
        # and add the gateway amount.
        if not self.context.r_company:
            self.context.net_amt = self.context.amount_auth
            self.context.amount_auth += self.gateway_amt
            self.context.gateway_amt = self.gateway_amt
            self.context.r_company = u'interswitch'
        self.amount_auth = int(100 * self.context.amount_auth)
        xmldict = {}
        provider_amt = 0.0
        self.pay_item_id = '101'
        xmldict['institution_acct'] = '1011005811'
        xmldict['institution_bank_id'] = '117'
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 100 * self.context.net_amt
        if not self.context.provider_amt:
            self.context.provider_amt = provider_amt
            self.context.amount_auth += provider_amt
        if provider_amt == 0:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.amount_auth = int(100 * self.context.amount_auth)
        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentVerifyWebservicePageStudent(
    InterswitchPaymentVerifyWebservicePageStudent):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentVerifyWebservicePageApplicant(
    InterswitchPaymentVerifyWebservicePageApplicant):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC


# PAYDirect added on 19/05/2021

PAYDIRECT_HOST = 'orion.interswitchng.com'
PAYDIRECT_URL = '/bookonhold/bookonhold.asmx'
MERCHANT_ID = '8124'

PAYDIRECTCATS = {
    }

class CustomPAYDirectActionButtonStudent(PAYDirectActionButtonStudent):

    @property
    def target_url(self):
        if self.context.p_category not in PAYDIRECTCATS.keys():
            return ''
        super(CustomPAYDirectActionButtonStudent, self).target_url
        return

class CustomPAYDirectActionButtonApplicant(PAYDirectActionButtonApplicant):

    @property
    def target_url(self):
        return ''

class CustomPAYDirectPageStudent(PAYDirectPageStudent):
    """Inform student how to proceed
    """
    grok.context(ICustomStudentOnlinePayment)
    gateway_amt = GATEWAY_AMT
    merchant_id = MERCHANT_ID
    gateway_url = PAYDIRECT_URL
    gateway_host = PAYDIRECT_HOST
    https = True

class CustomPAYDirectPageApplicant(PAYDirectPageApplicant):
    """ Inform applicant how to proceed.
    """
    grok.context(ICustomApplicantOnlinePayment)
    gateway_amt = GATEWAY_AMT
    merchant_id = MERCHANT_ID
    gateway_url = PAYDIRECT_URL
    gateway_host = PAYDIRECT_HOST
    https = True

class CustomStudentRefNumberSlip(StudentRefNumberSlip):
    """Deliver a PDF slip of the context.
    """
    merchant_id = MERCHANT_ID

class CustomApplicantRefNumberSlip(ApplicantRefNumberSlip):
    """Deliver a PDF slip of the context.
    """
    merchant_id = MERCHANT_ID
